'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _recast = require('recast');

var recast = _interopRequireWildcard(_recast);

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

var _stripIndent = require('strip-indent');

var _stripIndent2 = _interopRequireDefault(_stripIndent);

var _commonTags = require('common-tags');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

module.exports = pluginTester;

var fullDefaultConfig = {
  parserOpts: { parser: recast.parse },
  generatorOpts: { generator: recast.print, lineTerminator: '\n' },
  babelrc: false
};

function pluginTester() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref$plugin = _ref.plugin,
      plugin = _ref$plugin === undefined ? requiredParam('plugin') : _ref$plugin,
      _ref$pluginName = _ref.pluginName,
      pluginName = _ref$pluginName === undefined ? getPluginName(plugin) : _ref$pluginName,
      _ref$title = _ref.title,
      describeBlockTitle = _ref$title === undefined ? pluginName : _ref$title,
      tests = _ref.tests,
      fixtures = _ref.fixtures,
      rest = _objectWithoutProperties(_ref, ['plugin', 'pluginName', 'title', 'tests', 'fixtures']);

  var testAsArray = toTestArray(tests);
  if (!testAsArray.length) {
    return;
  }
  var testerConfig = (0, _lodash2.default)({}, fullDefaultConfig, rest);

  describe(describeBlockTitle, function () {
    testAsArray.forEach(function (testConfig, index) {
      if (!testConfig) {
        return;
      }

      var _merge = (0, _lodash2.default)({}, testerConfig, toTestConfig({ testConfig, index, plugin, pluginName, fixtures })),
          modifier = _merge.modifier,
          title = _merge.title,
          code = _merge.code,
          babelOptions = _merge.babelOptions,
          output = _merge.output,
          snapshot = _merge.snapshot;

      if (modifier) {
        it[modifier](title, tester);
      } else {
        it(title, tester);
      }

      function tester() {
        (0, _invariant2.default)(code, _commonTags.oneLine`
            A string or object with a \`code\` or
            \`fixture\` property must be provided
          `);
        if (snapshot) {
          (0, _invariant2.default)(!output, '`output` cannot be provided with `snapshot: true`');
        }
        var result = babel.transform(code, babelOptions).code.trim();
        if (snapshot) {
          (0, _invariant2.default)(result !== code, _commonTags.oneLine`
              Code was unmodified but attempted to take a snapshot.
              If the code should not be modified, set \`snapshot: false\`
            `);
          var separator = '\n\n      ↓ ↓ ↓ ↓ ↓ ↓\n\n';
          var formattedOutput = [code, separator, result].join('');
          expect(`\n${formattedOutput}\n`).toMatchSnapshot(title);
        } else if (output) {
          _assert2.default.equal(result, output, 'Output is incorrect.');
        } else {
          _assert2.default.equal(result, code, 'Expected output to not change, but it did');
        }
      }
    });
  });
}

function toTestArray(tests) {
  tests = tests || []; // null/0/false are ok, so no default param
  if (Array.isArray(tests)) {
    return tests;
  }
  return Object.keys(tests).reduce(function (testsArray, key) {
    var value = tests[key];
    if (typeof value === 'string') {
      value = { code: value };
    }
    testsArray.push(_extends({
      title: key
    }, value));
    return testsArray;
  }, []);
}

function toTestConfig(_ref2) {
  var testConfig = _ref2.testConfig,
      index = _ref2.index,
      plugin = _ref2.plugin,
      pluginName = _ref2.pluginName,
      fixtures = _ref2.fixtures;

  if (typeof testConfig === 'string') {
    testConfig = { code: (0, _stripIndent2.default)(testConfig).trim() };
  }
  var _testConfig = testConfig,
      title = _testConfig.title,
      fixture = _testConfig.fixture,
      _testConfig$code = _testConfig.code,
      code = _testConfig$code === undefined ? getCode(fixtures, fixture) : _testConfig$code,
      _testConfig$fullTitle = _testConfig.fullTitle,
      fullTitle = _testConfig$fullTitle === undefined ? `${index + 1}. ${title || pluginName}` : _testConfig$fullTitle,
      _testConfig$output = _testConfig.output,
      output = _testConfig$output === undefined ? getCode(fixtures, testConfig.outputFixture) : _testConfig$output;

  return (0, _lodash2.default)({}, testConfig, {
    babelOptions: { plugins: [plugin] },
    title: fullTitle,
    code: code.trim(),
    output: output.trim()
  });
}

function getCode(fixtures, fixture) {
  if (!fixture) {
    return '';
  }
  var fullPath = fixture;
  if (!_path2.default.isAbsolute(fixture)) {
    fullPath = _path2.default.join(fixtures, fixture);
  }
  return _fs2.default.readFileSync(fullPath, 'utf8');
}

function requiredParam(name) {
  throw new Error(`${name} is a required parameter.`);
}

function getPluginName(plugin) {
  var name = void 0;
  try {
    name = plugin(babel).name;
  } catch (error) {
    // eslint-disable-next-line no-console
    console.error(_commonTags.oneLine`
        Attempting to infer the name of your plugin failed.
        Tried to invoke the plugin which threw the error.
      `);
    throw error;
  }
  (0, _invariant2.default)(name, 'The `pluginName` must be inferable or provided.');
  return name;
}